<?php
// $Id: watcher.db.inc,v 1.1.2.1 2008/12/12 22:53:39 solipsist Exp $
/**
 * Watcher Module
 * by Jakob Persson of imBridge <jakob@imbridge.come>
 * 
 * Module allows users to watch nodes and receive notifications when nodes
 * are updated or commented on.
 * 
 * Sponsored by
 *   Wordsy - www.wordsy.com
 *   Webbredaktören - www.webbredaktoren.se
 */
 
/***************************************************************************************
 * DATABASE FUNCTIONS
 ***************************************************************************************/

/**   
 * Returns the nodes a user is watching
 * 
 * @param $uid
 *    The uid for the user whose binder we want posts for
 * @param $header
 *    The table header array, used for table sorting
 */
function _watcher_db_get_watched_nodes($uid = false, $header = null) { 
// Taken from tracker.module
// TODO: These queries are very expensive, see http://drupal.org/node/105639
  if ($uid) {
    $sql = 'SELECT DISTINCT(n.nid), n.title, n.type, n.changed, n.uid, u.name, wn.send_email, wn.added, GREATEST(n.changed, l.last_comment_timestamp) AS last_updated, l.comment_count FROM {node} n INNER JOIN {node_comment_statistics} l ON n.nid = l.nid INNER JOIN {users} u ON n.uid = u.uid INNER JOIN {watcher_nodes} wn ON n.nid = wn.nid LEFT JOIN {comments} c ON n.nid = c.nid AND (c.status = %d OR c.status IS NULL) WHERE n.status = 1 AND wn.uid = %d';
    if ($header) {
      $sql .= tablesort_sql($header);
    }
    $sql = db_rewrite_sql($sql);    
    $sql_count = 'SELECT COUNT(DISTINCT(n.nid)) FROM {node} n LEFT JOIN {comments} c ON n.nid = c.nid AND (c.status = %d OR c.status IS NULL) INNER JOIN {watcher_nodes} wn ON n.nid = wn.nid WHERE n.status = 1 AND wn.uid = %d AND wn.uid = %d';
    $sql_count = db_rewrite_sql($sql_count);
    $result = pager_query($sql, 25, 0, $sql_count, COMMENT_PUBLISHED, $uid, $uid, $uid);  
  }
  return $result;

}

/**
 * Get NIDs by UID
 * see what posts a user is watching
 * 
 * @param $uid
 *     UID - a user id.
 * @return
 *     A database result set of matching table rows.
 */
function _watcher_db_get_nids_by_uid($uid) {
  $sql = 'SELECT nid FROM {watcher_nodes} WHERE uid = %d';
  $result = db_query($sql, $uid);
  if (db_num_rows($result)>0) return $result;
  return false;
}

/**
 * Get UIDs by NID
 * see what users are watching a post
 * 
 * @param $uid
 *     UID - a user id.
 * @return
 *     A database result set of matching table rows.
 */
function _watcher_db_get_uids_by_nid($nid) {
  $sql = 'SELECT uid FROM {watcher_nodes} WHERE nid = %d';
  $result = db_query($sql, $nid);
  if (db_num_rows($result)>0) return $result;
  return false; 
}

/**
 * Is UID watching NID
 * check if a specific user is watching the given node
 * 
 * @param $uid
 *     UID - a user id.
 * @return
 *     A database result set of matching table rows.
 */
function _watcher_db_get_uid_nid($uid, $nid) {
  $sql = 'SELECT uid FROM {watcher_nodes} WHERE uid = %d AND nid = %d';
  $result = db_query($sql, $uid, $nid);
  if (db_num_rows($result)>0) return $result;
  return false; 
}

/**
 * Add NID to UID's watch list, enable email notifications if desired
 * 
 * @param $nid
 *     The nid of the node the users is going to watch
 * @param $uid
 *     The uid of the user
 * @param $send_email
 *     A boolean flag indicating whether the user will receive email notifications
 * @return
 *     Boolean true on success
 */
function _watcher_db_set_nid_uid($nid, $uid, $send_email = 0) {
  $sql = 'INSERT INTO {watcher_nodes} (uid, nid, send_email, added) VALUES(%d, %d, %d, %d)';
  $result = db_query($sql, $uid, $nid, $send_email, time());
  if ($result) return true;
  return false;
}

/**
 * Remove NID from the UID's watch list
 * @param $uid
 *     UID - a user id.
 * @param $nid
 *     NID - a node id.
 * @return
 *     A boolean indicating success/failure
 */
function _watcher_db_delete_nid_uid($nid, $uid) {
  $sql = 'DELETE FROM {watcher_nodes} WHERE uid = %d AND nid = %d';
  $result = db_query($sql, $uid, $nid);
  if ($result) return true;
  return false;
}

/**
 * Remove all NIDs on UID's watch list
 * @param $uid
 *     UID - a user id.
 * @return
 *     A boolean indicating success/failur
 */
function _watcher_db_delete_uid($uid) {
  $sql = 'DELETE FROM {watcher_nodes} WHERE uid = %d';
  $res = db_query($sql, $uid);
  return (bool) $res;
}

/**
 * Get email notification status given UID and NID
 * 
 * @param $uid
 *     UID - a user id.
 * @param $nid
 *     NID - a node id.
 * @return
 *     Boolean True if email notifications are enabled for this user and node
 */
function _watcher_db_get_send_email_status($nid, $uid) {
  $sql = 'SELECT send_email FROM {watcher_nodes} WHERE uid = %d AND nid = %d';
  $result = db_query($sql, $uid, $nid);
  $row = db_fetch_object($result);
  if ($row->send_email) return true;
  return false;
}

/**
 * Set email notification status given NID and UID
 *
 * @param $nid
 *     NID - a node id
 * @param $uid
 *     UID - a user id.
 * @param $send_email
 *     A boolean integer, 1 or 0
 * @return
 *     Boolean denoting success/failure.
 */
function _watcher_db_set_send_email_status($nid, $uid, $send_email = 0) {
  $sql = 'UPDATE {watcher_nodes} SET send_email = %d WHERE uid = %d AND nid = %d';
  $result = db_query($sql, $send_email, $uid, $nid);
  if ($result) return true;
  return false;
}

/**
 * Save user settings to database
 * 
 * @param $uid
 *     A user UID
 * @param $settings
 *     An associative array of user settings
 * @return
 *     Boolean true on success
 */
function _watcher_db_set_user_settings($uid, $settings) {

  //Add custom field to mark these user settings as custom
  $settings['custom'] = 1;
  
  //Serialize settings
  $fields = implode(', ', array_keys($settings));
  $values = implode(', ', array_values($settings));
  
  // First delete existing settings
  $sql = 'DELETE FROM {watcher_user_settings} WHERE uid = %d';
  db_query($sql, $uid);
  
  // Insert new settings
  $sql = 'INSERT INTO {watcher_user_settings} (uid, %s) VALUES (%d, %s)';
  $res = db_query($sql, $fields, $uid, $values);
  return (bool) $res;
}

/**
 * Delete user settings for given UID
 * 
 * @param $uid
 *     A user UID
 * @return
 *     Boolean True on success
 */
function _watcher_db_delete_user_settings($uid) {
  $sql = 'DELETE FROM {watcher_user_settings} WHERE uid = %d';
  $res = db_query($sql, $uid);
  return (bool) $res;
}

/**
 * Load user settings from database
 * 
 * @param $uid
 *     A user UID
 * @return
 *     An assoc array of user settings
 */
function _watcher_db_get_user_settings($uid) {
  // Cache settings
  static $settings;
  if (!isset($settings)) {
    $sql = 'SELECT * FROM {watcher_user_settings} WHERE uid = %d';
    if (($res = db_query($sql, $uid)) && (db_num_rows($res) == 1)) {
      $row = db_fetch_array($res);
      unset($row['uid']);
      $settings = $row;
    } 
    else {
      $settings = false;
    }
  }
  return $settings;
}

/**
 * Updates the default settings for all users who haven't customized their settings
 * 
 * @param $settings
 *    An associative array of settings
 * @return
 *    A boolean true on success
 */
function _watcher_db_user_settings_update_defaults($settings) {
  // Users may have registered and not had settings set yet so add rows for any users that are "missing"
  $sql = 'INSERT INTO {watcher_user_settings} (uid) SELECT u.uid FROM {users} u LEFT JOIN {watcher_user_settings} wus ON u.uid = wus.uid WHERE ISNULL(custom)';
  $res = db_query($sql);

  // If defaults are set, apply them, otherwise return
  if (is_array($settings)) {
    // Format settings array as string
    foreach ($settings as $setting => $value) {
      $insert_array[] = $setting .'='. $value;      
    }
    $insert_string = implode(', ', $insert_array);

    // Update all users who haven't customized their settings yet
    $sql = 'UPDATE {watcher_user_settings} SET %s WHERE custom <> 1';
    $res = db_query($sql, $insert_string);
  }
  return (bool) $res;   
}

/**
 * Has UID customized his/her user settings?
 * 
 * @param $uid
 *    A user UID
 * @return
 *    A boolean true if given UID has customized his/her user settings
 */
function _watcher_db_user_settings_uid_custom($uid) {
  $sql = 'SELECT custom FROM {watcher_user_settings} WHERE uid = %d AND custom = 1';
  $res = db_query($sql, $uid);
  return (bool) db_num_rows($res);
}

/**
 * Return a list of users that want to get notified about node updates for given NID
 * 
 * @param $nid 
 *     The NID of the node that has been updated
 * @param $uid_exclude
 *     A UID we want to exclude from our set of users, normally the user that updated the node.
 * @return
 *     An array of objects with uid as key representing users, with attributes uid, mail and name
 */
function _watcher_db_get_users_notify_node_update($nid, $uid_exclude) {
  $sql = "SELECT wn.uid, u.mail, u.name FROM {watcher_nodes} wn INNER JOIN {watcher_user_settings} wus ON wus.uid = wn.uid INNER JOIN {users} u ON u.uid = wus.uid WHERE notifications_updates = 1 AND send_email = 1 AND nid = %d AND wn.uid <> %d;";
  $res = db_query($sql, $nid, $uid_exclude);
  $users = array();
  while ($row = db_fetch_object($res)) {
    $users[$row->uid] = $row;
  }
  return $users;
}

/**
 * Return a list of users that want to get notified about new comments for given NID
 * 
 * @param $nid
 *     The nid of the node the comment belongs to
 * @param $uid_exclude 
 *     A uid we want to exclude from our set of users, normally the user that wrote the comment,
 *     what's the point in being notified about your own comments, eh?
 * @return
 *     An array of objects with uid as key representing users, with attributes uid, mail and name
 */
function _watcher_db_get_users_notify_comment_insert($nid, $uid_exclude) {
  $sql = "SELECT wn.uid, u.mail, u.name FROM {watcher_nodes} wn INNER JOIN {watcher_user_settings} wus ON wus.uid = wn.uid INNER JOIN {users} u ON u.uid = wus.uid WHERE notifications_new_comments = 1 AND send_email = 1 AND nid = %d AND wn.uid <> %d;";
  $res = db_query($sql, $nid, $uid_exclude);
  $users = array();
  while ($row = db_fetch_object($res)) {
    $users[] = $row;
  }
  return $users;
}

/**
 * Return a list of users that want to get notified about node updates or new comments
 * for a given NID
 * 
 * @param $nid 
 *      The nid of the node that has been updated
 * @param $uid_exclude
 *      A uid we want to exclude from our set of users, normally the user that updated the node.
 * @return
 *      An array of objects with uid as key representing users, with attributes uid, mail and name
 */
function _watcher_db_get_users_notify_about_node($nid, $uid_exclude) {
  $sql = "SELECT wn.uid, u.mail, u.name FROM {watcher_nodes} wn INNER JOIN {watcher_user_settings} wus ON wus.uid = wn.uid INNER JOIN {users} u ON u.uid = wus.uid WHERE nid = %d AND wn.uid <> %d AND send_email = 1";
  $res = db_query($sql, $nid, $uid_exclude);
  $users = array();
  while ($row = db_fetch_object($res)) {
    $users[$row->uid] = $row;
  }
  return $users;
}

/**
 * Add a message to the notification queue table
 * 
 * @param $qid
 *     A qid (table pk) generated by db_next_id()
 * @param $r_uid
 *     Recipient uid
 * @param $r_email
 *     Recipient email address
 * @param $subject
 *     The message subject
 * @param $message
 *     Message to be sent
 * @return
 *     A database result resource
 */
function _watcher_db_insert_users_notification_queue($qid, $r_uid, $r_email, $subject, $message) {
  $sql = "INSERT INTO {watcher_notify_queue} (qid, uid, mail, subject, message) VALUES (%d, %d, '%s', '%s', '%s')";
  $res = db_query($sql, $qid, $r_uid, $r_email, $subject, $message);
  return (bool) $res;
}

/**
 * Delete all rows in notification queue table
 * 
 * @return
 *    A boolean denoting whether any rows were deleted, false if none, true if one or more
 */
function _watcher_db_delete_users_notification_queue_all() {
  $sql = "DELETE FROM {watcher_notify_queue}";
  $res = db_query($sql);
  return (bool) $res;
}

/**
 * Get messages from the notification queue table
 */
function _watcher_db_get_users_notification_queue() {
  $sql = "SELECT * FROM {watcher_notify_queue}"; 
  $res = db_query($sql);
  while ($qmsg = db_fetch_object($res)) {
    $qmsgs[] = $qmsg;
  }
  return $qmsgs;
}

/**
 * Delete a set of rows from the notification queue table
 * 
 * @param $rows
 *     An array of qids (queue id)
 */
function _watcher_db_delete_users_notification_queue($qids) {

  // Database managers can only handle *finite* (gulp!) SQL queries 
  // so we gotta limit the fun
  $limit = 4096; //2^12 elements
  
  $offset = 0;
  for ($i = 1; $offset < count($qids); $i++) {
    $batch = array_slice($qids, $offset, $limit);
    $in = implode(',', $batch);
    $sql = "DELETE FROM {watcher_notify_queue} WHERE qid IN (%s)";
    db_query($sql, $in);
    $offset = (($i * $limit) - 1);
  }
}

/**
 * Statistics for Notification Queue Table
 * 
 * @return
 *    An assoc array with keys num_rows and num_sent
 */
function _watcher_db_stats_notification_queue() {
  $sql = 'SELECT * FROM {watcher_notify_queue}';
  $sql2 = "SELECT id FROM {sequences} WHERE name = '{watcher_notify_queue}_qid'";
  
  // Obtain number of rows
  $res = db_query($sql);
  $stat['num_rows'] = db_num_rows($res);
  
  // Obtain the contents of the queue table
  while ($row = db_fetch_object($res)) {
    
    // Load fields
    $data['qid'] = $row->qid;
    $data['uid'] = $row->uid;
    $data['mail'] = $row->mail;
    $data['subject'] = $row->subject;
    $data['message'] = '<pre>'. $row->message .'</pre>';
    
    // Add to data array
    $stat['data']['content'][] = $data;
  }
  
  // Add header
  $stat['data']['header'] = array('qid', 'uid', 'mail', 'subject', 'message');
  
  // Obtain the number of notifications sent - this is not 100% accurate since we cannot
  // entirely trust the increment counter
  $res2 = db_query($sql2);
  $row = db_fetch_object($res2);
  $stat['num_sent'] = ( $row->id > 0 ? $row->id - $stat['num_rows'] : 0 );
  
  if ($res) {
    return $stat;
  }
  return false;
}

/**
 * Statistics for Watched Nodes Table
 * 
 * @return
 *    An assoc array with key num_users
 */
function _watcher_db_stats_watched_nodes_users() {
  $sql = 'SELECT uid FROM {watcher_nodes} GROUP BY(uid)';
  
  // Obtain number of rows
  $res = db_query($sql);
  $stat['num_users'] = db_num_rows($res);
  
  if ($res) return $stat;
  return false;
}

/**
 * Statistics for User Settings Table
 * 
 * @return
 *    An assoc array with key num_users_with_settings, num_users and num_users_no_settings
 */
function _watcher_db_stats_user_settings() {
  
  // Users with custom settings
  $sql = 'SELECT uid FROM {watcher_user_settings} WHERE custom = 1';
  
  // Total number of users on the site
  $sql2 = 'SELECT uid FROM {users} WHERE uid <> 0';
  
  // Obtain number of users with settings
  $res = db_query($sql);
  $stat['num_users_with_settings'] = db_num_rows($res);
  
  // Obtain number of users without setting
  $res2 = db_query($sql2);
  $stat['num_users'] = db_num_rows($res2);  
  $stat['num_users_no_settings'] = $stat['num_users'] - $stat['num_users_with_settings'];
  
  if ($res) return $stat;
  return false;
}

/**
 * Return the newest node
 */
function _watcher_db_get_newest_node() {
  $sql = 'SELECT nid FROM {node} ORDER BY created DESC';
  $res = db_query($sql);
  if (!db_num_rows($res)) {
    return false;
  }
  return db_result($res);    
}

/**
 * Return the newest comment
 */
function _watcher_db_get_newest_comment() {
  $sql = 'SELECT cid FROM {comments} ORDER BY timestamp DESC';
  $res = db_query($sql);
  if (!db_num_rows($res)) {
    return false;
  }
  return db_result($res);
}